#!/bin/sh

#set -x

# Copyright (c) 2003 Alexander Leidinger. All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.

# $FreeBSD$

# We want to use the installed perl per default, so check if just installed
# perl and set the new default.
check_perl() {
	# We just installed perl when we are in the perl port directory.
	if [ ${1} = /usr/ports/lang/perl5 ]; then
		echo ++++++++++ detected perl5
		use.perl port
	fi
}

# Build the port which is located in the given directory
# Handle all dependencies recursivly and build packages if necessary.
build_port() {
	# We get called ourself recursivly, so the list of dependend ports
	# has to be a local variable.
	local DEPENDS_LIST

	echo ++++++++++ no package detected, installing ${1} from port
	# We build the dependencies on our own to be able to
	# catch an implicit dependency on perl (to make it
	# the default perl for the system) and to build
	# packages for them (if requested).
	DEPENDS_LIST=$(make all-depends-list)
	[ -z "${DEPENDS_LIST}" ] || echo ++++++++++ ${1} depends on: ${DEPENDS_LIST}, installing those first
	for port in ${DEPENDS_LIST}; do
		install_port ${port}
	done

	# Remember the dependencies to be able to deinstall them at the end.
	BUILD_DEPENDS_LIST="${BUILD_DEPENDS_LIST} $(make build-depends-list)"
	PACKAGE_DEPENDS_LIST="${ALL_DEPENDS_LIST} $(make package-depends-list | cut -f 2 -d ' ')"

	# We are perhaps in a directory of the last dependency,
	# so we need to go back where we started.
	cd ${1}
	make install BATCH=yes ${MAKE_PACKAGE_TARGET} clean
}

# Install the port which is located in the given directory. If there is a
# package available for this port, use the package instead of building the
# port (again).
install_port() {
	echo ++++++++++ going to install ${1}
	# We call ourself recursivly, so the list of dependend ports has to
	# be a local variable.

	cd ${1}

	# We do not need to try to install again, when the port is already
	# installed.
	if [ ! -d /var/db/pkg/$(make -V PKGNAME) ]; then
		echo ++++++++++ installing ${1}

		# Check if there is a package for this port.
		if [ -f $(make -V PKGFILE) ]; then
			echo ++++++++++ package detected, installing ${1} from package
			pkg_add $(make -V PKGFILE) || build_port ${1}
		else
			build_port ${1}
		fi

		# In case we just installed perl we want to make it the
		# default perl of the system.
		check_perl ${1}
	else
		echo "  +-> skipping recursive build of ${1}, already installed"
	fi
}

# We do not want ports which are only used to build other ports, remove them.
delete_build_dependencies() {
	local PKGNAME UNIQUE_PACKAGE_DEPENDS_LIST UNIQUE_BUILD_DEPENDS_LIST

	echo '  +-> deleting build dependencies'
	UNIQUE_PACKAGE_DEPENDS_LIST="$(echo ${PACKAGE_DEPENDS_LIST} | sort -u)"
	UNIQUE_BUILD_DEPENDS_LIST="$(echo ${BUILD_DEPENDS_LIST} | sort -u)"

	echo ++++++++++ UNIQUE_BUILD_DEPENDS_LIST: ${UNIQUE_BUILD_DEPENDS_LIST}
	echo ++++++++++ UNIQUE_PACKAGE_DEPENDS_LIST: ${UNIQUE_PACKAGE_DEPENDS_LIST}

	for port in ${UNIQUE_BUILD_DEPENDS_LIST}; do
		# If the port is not in the list of really needed
		# ports, then remove it.

		echo ${UNIQUE_PACKAGE_DEPENDS_LIST} | grep -q ${port}
		if [ $? -ne 0 ]; then
			cd ${port}

			PKGNAME=$(make -V PKGNAME)

			if [ -d /var/db/pkg/${PKGNAME} ]; then
				echo "    +-> deleting ${PKGNAME}"
				pkg_delete ${PKGNAME}
			fi
		fi
	done
}


